import os
import shutil
import unittest
from os.path import exists

from digipathos.data_loader import DataLoader


class TestDataLoader(unittest.TestCase):
    def setUp(self):
        self.data_loader = DataLoader(lang='pt')

    def test_load_items(self):
        items = self.data_loader.get_datasets()
        self.assertIsNotNone(items)

    def test_load_crops(self):
        families = self.data_loader.get_crops()
        self.assertIsNotNone(families)

    def test_load_datasets_from_crop(self):
        families = self.data_loader.get_datasets_from_crop('Abacaxi')
        self.assertIsNotNone(families)

    def test_load_datasets_from_crop_invalid(self):
        self.assertRaises(ValueError, self.data_loader.get_datasets_from_crop, 'Jacaré')

    def test_get_dataset(self):
        item = self.data_loader.get_dataset(871)
        self.assertIsNotNone(item)

    def test_download_dataset(self):
        self.data_loader.download_dataset(871)

        output = os.path.join(self.data_loader.artifacts_path, '871.zip')
        self.assertTrue(exists(output))

        shutil.rmtree(self.data_loader.artifacts_path)

    def test_download_datasets_from_crop(self):
        self.data_loader.download_datasets_from_crop('Abacaxi')
        output = os.path.join(self.data_loader.artifacts_path, 'Abacaxi')
        count_dirs = len(os.listdir(output))
        self.assertTrue(count_dirs > 1)

        shutil.rmtree(self.data_loader.artifacts_path)



import unittest

from digipathos.constants import BASE_URL
from digipathos.dataset import Dataset


class TestDataset(unittest.TestCase):
    def setUp(self):
        self.dataset = Dataset(full_name='Abacaxi (Pineapple) - Broca (Pineapple Fruit Borer) - 1.zip',
                               url='/jspui/bitstream/123456789/871/4/Abacaxi%20%28Pineapple%29%20-%20Broca%20%28Pineapple%20Fruit%20Borer%29%20-%201.zip',
                               extension='zip',
                               size=1000)

    def test_name_english(self):
        expected_name = 'Pineapple'
        self.assertEqual(self.dataset.get_crop_name(), expected_name)

    def test_name_portuguese(self):
        expected_name = 'Abacaxi'
        self.assertEqual(self.dataset.get_crop_name(lang='pt'), expected_name)

    def test_name_invalid_lang(self):
        ex = 'cz'
        self.assertRaises(ValueError, self.dataset.get_crop_name, lang=ex)

    def test_disorder_invalid_lang(self):
        lang = 'cz'
        self.assertRaises(ValueError, self.dataset.get_disorder_name, lang=lang)

    def test_disorder_english(self):
        expected_disorder = 'Pineapple Fruit Borer'
        self.assertTrue(self.dataset.get_disorder_name(), expected_disorder)

    def test_disorder_portuguese(self):
        expected_disorder = 'Brota'
        self.assertTrue(self.dataset.get_disorder_name(lang='pt'), expected_disorder)

    def test_id(self):
        expected_id = 871
        self.assertEqual(self.dataset.get_id(), expected_id)

    def test_init_from_json(self):
        item = {}
        item[
            'bsLink'] = '/jspui/bitstream/123456789/871/4/Abacaxi%20%28Pineapple%29%20-%20Broca%20%28Pineapple%20Fruit%20Borer%29%20-%201.zip'
        item['name'] = 'Abacaxi (Pineapple) - Broca (Pineapple Fruit Borer) - 1.zip'
        item['format'] = 'zip'
        item['size'] = 1000

        full_url = BASE_URL + item['bsLink']
        other_dataset = Dataset.make_from_json_dict(item=item)

        self.assertEqual(other_dataset.url, full_url)
        self.assertEqual(other_dataset.full_name, item['name'])
        self.assertEqual(other_dataset.extension, item['format'])
        self.assertEqual(other_dataset.size, item['size'])



import os

from setuptools import find_packages, setup

HERE = os.path.abspath(os.path.dirname(__file__))


def read_file(*file_name: str) -> str:
    """
    This function is used to read a given file and return its contents
    :param file_name the file to be read
    :return: the string containing the file contents
    """
    with open(os.path.join(HERE, *file_name)) as f:
        return f.read()


setup(
    name='digipathos',
    version='0.1',
    license='BSD-3-Clause',
    url='https://github.com/bresan/digipathos_plant_pathology',
    description='',
    long_description=read_file('README.md'),
    long_description_content_type="text/markdown",
    author='Rodrigo Bresan',
    author_email='',
    classifiers=[
        'Programming Language :: Python :: 3.6',
    ],
    packages=find_packages(),
    install_requires=read_file('requirements.txt')
)



import random

from digipathos.data_loader import DataLoader

if __name__ == '__main__':
    data_loader = DataLoader(artifacts_path='artifacts')

    datasets = data_loader.get_datasets()

    crops = data_loader.get_crops()

    datasets_from_crop = data_loader.get_datasets_from_crop('Pineapple')

    # Not let's download a dataset
    dataset_id = random.choice(list(datasets.keys()))
    data_loader.download_dataset(dataset_id=dataset_id)

    data_loader.download_datasets_from_crop('Pineapple')
    data_loader.download_all_datasets()


from digipathos.cli.commands.command import Command, get_commands, transpose_and_print
from digipathos.data_loader import DataLoader

# print('[1] list_datasets')
# print('[2] list_crops')
# print('[3] list_datasets_from_crop')
# print('[4] get_dataset')
# print('[5] download_dataset')
# print('[6] download_all_datasets')
# print('[7] download_datasets_from_crop')
#
DEFAULT_LANGUAGE = 'en'

data_loader = DataLoader(auto_fetch=False, lang=DEFAULT_LANGUAGE)
commands = get_commands(data_loader)


def print_header():
    header = """  
    ____  _       _             __  __              
   / __ \(_)___ _(_)___  ____ _/ /_/ /_  ____  _____
  / / / / / __ `/ / __ \/ __ `/ __/ __ \/ __ \/ ___/
 / /_/ / / /_/ / / /_/ / /_/ / /_/ / / / /_/ (__  ) 
/_____/_/\__, /_/ .___/\__,_/\__/_/ /_/\____/____/  
        /____/ /_/                                  """

    print(header)


def init_interactive_mode():
    while True:
        try:
            print_header()
            command = read_command()
            if command.require_args:
                arg = input('%s: ' % command.args_description)

                if arg:
                    run_command_with_argument(command, arg)
                else:
                    print('You should provide an argument for this command')
            else:
                command.execute()
        except Exception:
            print("You should select a valid option from the command list")


def run_command_with_argument(command, arg):
    try:
        command.execute(arg)
    except Exception as e:
        print('There was an error on your command. Make sure your input is correct. Exception: ', e)


def read_command() -> Command:
    command_ids = ['ID']
    command_names = ['Command']
    for command_id, command in commands.items():
        command_ids.append(command_id)
        command_names.append(command.desc)

    commands_list = [command_ids, command_names]

    transpose_and_print(commands_list)

    try:
        command_id = int(input('Command: '))
        command = commands[command_id]
        return command
    except Exception:
        raise Exception()


if __name__ == '__main__':
    init_interactive_mode()
